name = 'lights'
version = '2.1'

speed = 2;
adjust = 5;  
mode = "twinkle"
set = "White"
delay = 400;
scale = .75;
hanging = false;

bulb = new Image();
corner = new Image();

topContext = null;
bottomContext = null;
leftContext = null;
rightContext = null;
topLeftContext = null;
topRightContext = null;

lightsetPath = ""
refreshCorners = true;
visible = true;
c = 0;

updateReq = null;

function setup() {

	//initalize lightset directory if it doesn't exist
	uid = widget.system("id | cat", null).outputString;
	uid = uid.match(/uid=\d*\((.*?)\)/)[1];
	lightsetPath = "file:///Users/" + uid + "/Library/Application Support/Festive Lights/Lightsets/";

	ensureDirectoryExists("~/Library/Application\\ Support/Festive\\ Lights/");
	if(!pathExists("~/Library/Application\\ Support/Festive\\ Lights/Lightsets"))
		widget.system("cp -R Lightsets ~/Library/Application\\ Support/Festive\\ Lights/", null);
	
	//preliminary adjustments
	//window.resizeTo(screen.width, screen.height);
	widget.resizeAndMoveTo(-1, -1, screen.width + 2, screen.height + 2);
	widget.setCloseBoxOffset(137, 43);
	document.getElementById('lights').style.width = screen.width + 2;
	document.getElementById('lights').style.height = screen.height + 2;
	checkForUpdate();
	
	loadLightSets();
    loadPreferences();

	//initialize the drawing contexts
    topContext = document.getElementById("top").getContext("2d");
    bottomContext = document.getElementById("bottom").getContext("2d");
    leftContext = document.getElementById("left").getContext("2d");
    rightContext = document.getElementById("right").getContext("2d");
    
    topLeftContext = document.getElementById("top_left").getContext("2d");
    topRightContext = document.getElementById("top_right").getContext("2d");
    bottomLeftContext = document.getElementById("bottom_left").getContext("2d");
    bottomRightContext = document.getElementById("bottom_right").getContext("2d");
    
 	switchLightSet(set);
 	switchMode(mode);
 	adjustScale(scale);
	
	drawLights();
}

function drawLights()
{
    if(visible)
    {   
    	//redraw the corners when necessary
    	if(refreshCorners) drawCorners();
    
        //save the intial contexts
        topContext.save();
        bottomContext.save();
        leftContext.save();
        rightContext.save();
                
        //transform the contexts appropriately
        bottomContext.translate(screen.width - ((hanging) ? 0 : 2*bulb_h_scaled), bulb_h_scaled);
        bottomContext.rotate(3.14);
        
        rightContext.translate(bulb_h_scaled, 0);
        rightContext.rotate(1.57);
        
        leftContext.translate(0, screen.height - 2*bulb_h_scaled);
        leftContext.rotate(-1.57);
        
        //clear the contexts
        topContext.clearRect(0, 0, screen.width, bulb_h_scaled);
        bottomContext.clearRect(0, 0, screen.width, bulb_h_scaled);
        leftContext.clearRect(0, 0, screen.height, bulb_h_scaled);
        rightContext.clearRect(0, 0, screen.height, bulb_h_scaled);
        
        x = 0;
        n = 0;
        
        //draw the strands
        while(x < screen.width - ((hanging) ? 0 : 2*bulb_h_scaled))
        {
            var state = "off/";
            var b = n%n_bulbs;
            
            //use some clever math to turn the next bulb on or off...
            if(mode == "blink")
                state = (((n + (adjust+1)*(c%2))%((adjust+1)*2) < (adjust+1)) ? "on/" : "off/");
            else if(mode == "chase")
                state = (((n-c)%(3+adjust) == 0) ? "on/" : "off/");
            else if(mode == "twinkle")
                state = ((Math.random() > (11-adjust)/12) ? "on/" : "off/");
            else if(mode == "on")
                state = "on/";
            
            //load the bulb graphic
            bulb.src = lightsetPath + set + "/" + state + "bulb_" + b + ".png";
            
            topContext.drawImage(bulb, x, 0, bulb_w_scaled, bulb_h_scaled)
            bottomContext.drawImage(bulb, x, 0, bulb_w_scaled, bulb_h_scaled)
            
            if(x < screen.height - 2*bulb_h_scaled)
            {
                leftContext.drawImage(bulb, x, 0, bulb_w_scaled, bulb_h_scaled)
                rightContext.drawImage(bulb, x, 0, bulb_w_scaled, bulb_h_scaled)
            }
            
            x += bulb_w_scaled;
            n++;
        }
        
        //restore the initial contexts
        topContext.restore();
        bottomContext.restore();
        leftContext.restore();
        rightContext.restore();
    
    c++;
    }
    
    setTimeout("drawLights()", delay);
}

function drawCorners()
{
	refreshCorners = false;

	//load the corner graphic
	corner.src = lightsetPath + set + "/corner.png";
	
	//draw corners once graphic loads
	corner.onload = function() {
		
		//save the intial contexts
		topLeftContext.save();
		topRightContext.save();
		bottomLeftContext.save();
		bottomRightContext.save();
		
		//transform contexts appropriately
		topRightContext.translate(bulb_h_scaled, 0);
		topRightContext.rotate(1.57);
		
		bottomRightContext.translate(bulb_h_scaled, bulb_h_scaled);
		bottomRightContext.rotate(-3.14);
		
		bottomLeftContext.translate(0, bulb_h_scaled);
		bottomLeftContext.rotate(-1.57);
		
		//clear the contexts
		topLeftContext.clearRect(0, 0, bulb_h_scaled, bulb_h_scaled);
		bottomLeftContext.clearRect(0, 0, bulb_h_scaled, bulb_h_scaled);
		topRightContext.clearRect(0, 0, bulb_h_scaled, bulb_h_scaled);
		bottomRightContext.clearRect(0, 0, bulb_h_scaled, bulb_h_scaled);
	
		//draw the corners
		topLeftContext.drawImage(corner, 0, 0, bulb_h_scaled, bulb_h_scaled);
		topRightContext.drawImage(corner, 0, 0, bulb_h_scaled, bulb_h_scaled);
		bottomLeftContext.drawImage(corner, 0, 0, bulb_h_scaled, bulb_h_scaled);
		bottomRightContext.drawImage(corner, 0, 0, bulb_h_scaled, bulb_h_scaled);
		
		//restore the initial contexts
		topLeftContext.restore();
		topRightContext.restore();
		bottomLeftContext.restore();
		bottomRightContext.restore();
	}
}

function loadPreferences() {
    set = widget.preferenceForKey("set");
    mode = widget.preferenceForKey("mode");
    scale = widget.preferenceForKey("scale");
    
    twinkle_speed = widget.preferenceForKey("twinkle_speed");
    blink_speed = widget.preferenceForKey("blink_speed");
    chase_speed = widget.preferenceForKey("chase_speed");

    density = widget.preferenceForKey("density");
    grouping = widget.preferenceForKey("grouping");
    spacing = widget.preferenceForKey("spacing");

	hanging = widget.preferenceForKey("hanging");    
    if(hanging) document.getElementById("hanging").checked = true;

    if(set != undefined) setPopupSelection(document.getElementById('set'), set);
    else set = "White";
    
    if(mode != undefined) setPopupSelection(document.getElementById('mode'), mode);
    else mode = "twinkle"
    
    if(scale != undefined) document.getElementById('scale').value = scale*75;
    else scale = .75;

    if(blink_speed == undefined) blink_speed = 3;
    if(chase_speed == undefined) chase_speed = 5;
    if(twinkle_speed == undefined) twinkle_speed = 2;
        
    if(grouping == undefined) grouping = 0;
    if(spacing == undefined) spacing = 2;
    if(density == undefined) density = 5;
}

function loadLightSets() {
    builtinSets = "White Colored";
    numCustomSets = 0;

    sets = widget.system("ls -F ~/Library/Application\\ Support/Festive\\ Lights/Lightsets | grep \"/\"", null).outputString;
    sets = sets.split("/\n");
    
    HTML_begin = "<select id='set' style='width:115' onChange='switchLightSet(this.options[this.selectedIndex].value)'>";
    HTML = "<option value='White'>White</option>";
    HTML += "<option value='Colored'>Colored</option>";
    
    for(i = 0; i < sets.length-1; i++) {
        if(!builtinSets.match(sets[i]))
        {
            numCustomSets++;
            
            if(numCustomSets == 1)
                HTML += "<\optgroup><optgroup label='Custom'>";
            
            HTML += "<option value='" + sets[i] + "'>" + sets[i] + "</option>";
        }
    }
    
    if(numCustomSets > 0)
        HTML = "<optgroup label='Standard'>" + HTML + "</optgroup>";
    
    document.getElementById('lightSets').innerHTML = HTML_begin + HTML + "</select>";
}

function switchLightSet(newSet) {
    try {
    
        oldSet = set;
        set = newSet.replace(' ', '\\ ');
        
        //determine bulb width
        bulb_w = widget.system("sips -g pixelWidth ~/Library/Application\\ Support/Festive\\ Lights/Lightsets/" + set + "/on/bulb_0.png", null).outputString;
        bulb_w = 1*bulb_w.replace(/.*\n.*Width:[^\d]*(\d*).*/m,"$1");
        bulb_w_scaled = bulb_w*scale;
        
        //determine bulb height
        bulb_h = widget.system("sips -g pixelHeight ~/Library/Application\\ Support/Festive\\ Lights/Lightsets/" + set + "/on/bulb_0.png", null).outputString
        bulb_h = 1*bulb_h.replace(/.*\n.*Height:[^\d]*(\d*).*/m,"$1");
        bulb_h_scaled = bulb_h*scale
        
        //determine number of bulbs in the set
        n_bulbs = widget.system("ls ~/Library/Application\\ Support/Festive\\ Lights/Lightsets/" + set + "/on | wc", null).outputString;
        n_bulbs = n_bulbs.replace(/[^\d]*(\d*).*/m,"$1");
        
        //check for hanging-only lightsets
        hanging = (!pathExists("~/Library/Application\\ Support/Festive\\ Lights/Lightsets/" + set + "/corner.png"));
        
        document.getElementById("hanging").disabled = hanging;
		if(widget.preferenceForKey("hanging") == true) hanging = true;
        document.getElementById("hanging").checked = hanging;
        
        set = newSet;
        
        adjustScale(scale);
        
        widget.setPreferenceForKey(set, "set");
        return true;
    }
    catch (ex)
    {
        alert("Couldn't load light set \"" + set + "\"." + ex);
        
        if(oldSet == set) switchLightSet("White");
        else switchLightSet(oldSet);
        
        return false;
    }
}

function switchMode(newMode) {
    mode = newMode;
    widget.setPreferenceForKey(mode, "mode");
    
    document.getElementById('sliders').style.display = 'block';
    
    if(mode == "blink") {
        document.getElementById('adjust').innerHTML = '<td>Grouping:</td>' + "<td><input type='range' id='behavior' style='width:60' min='0' max='10' step='1' oninput='adjustBehavior(this.value*1)' /></td>";
        speed = blink_speed;
        adjust = grouping;
        delay = 300*speed;
    } else if(mode == "chase") {
        document.getElementById('adjust').innerHTML = '<td>Spacing:</td>' + "<td><input type='range' id='behavior' style='width:60' min='0' max='10' step='1' oninput='adjustBehavior(this.value*1)' /></td>";
        speed = chase_speed;
        adjust = spacing;
        delay = 50*speed;
    } else if(mode == "twinkle") {
        document.getElementById('adjust').innerHTML = '<td>Density:</td>' + "<td><input type='range' id='behavior' style='width:60' min='0' max='10' step='1' oninput='adjustBehavior(this.value*1)' /></td>";
        speed = twinkle_speed;
        adjust = density;
        delay = 200*speed;
    } else {
        document.getElementById('sliders').style.display = 'none';
        delay = 500;
	}
    
    document.getElementById('speed').value = 20*(5.05 - speed);
    document.getElementById('behavior').value = adjust;
}

function adjustSpeed(value) {
    
    speed = 5.05 - value/20;
    
    if(mode == "blink") {
        blink_speed = speed;
        widget.setPreferenceForKey(speed, "blink_speed");
        delay = 300*speed;
    } else if(mode == "chase") {
        chase_speed = speed;
        widget.setPreferenceForKey(speed, "chase_speed");
        delay = 50*speed;
    } else if(mode == "twinkle") {
        twinkle_speed = speed;
        widget.setPreferenceForKey(speed, "twinkle_speed");
        delay = 200*speed;
    } else {
    	delay = 500;
    }
}

function adjustScale(value) {
    
    
    //store new scale value
    scale = value;
    widget.setPreferenceForKey(scale, "scale");
    
    //scale bulb dimensions
    bulb_w_scaled = bulb_w*scale;
    bulb_h_scaled = bulb_h*scale;
   	
   	//update drawing contexts
   	h = (hanging) ? 0 : bulb_h_scaled;
    document.getElementById("top_left").style.width = h;
    document.getElementById("top_left").style.height = h;
    document.getElementById("top_right").style.width = h;
    document.getElementById("top_right").style.height = h;
    document.getElementById("bottom_left").style.width = h;
    document.getElementById("bottom_left").style.height = h;
    document.getElementById("bottom_right").style.width = h;
    document.getElementById("bottom_right").style.height = h;
   
    document.getElementById("top").style.height = bulb_h_scaled;
    document.getElementById("bottom").style.height = h;
    document.getElementById("left").style.width = h;
    document.getElementById("right").style.width = h;
    
    //need to redraw corners
    refreshCorners = true;
}

function adjustBehavior(value) {
    
    adjust = value;
    
    if(mode == "blink") {
        grouping = value;
        widget.setPreferenceForKey(grouping, "grouping");
    } else if(mode == "chase") {
        spacing = value;
        widget.setPreferenceForKey(spacing, "spacing");
    } else if(mode == "twinkle") {
        density = value;
        widget.setPreferenceForKey(density, "density");
    }
}

function toggleHanging(on) {
	hanging = !hanging;
	adjustScale(scale);
	
	widget.setPreferenceForKey(hanging, "hanging");
}

function toggleControls() {
    controls = document.getElementById('controls');
    
    if(controls.style.display != 'block')
        controls.style.display = 'block';
    else
        controls.style.display = 'none';  
}

function setPopupSelection(popup, value)
{
    for(i = 0; i < popup.length; i++)
    {
        if (popup.options[i].value == value)
        {
            popup.options[i].selected = true;
            break;
        }
    }
}

function dragdrop (event)
{
    var uri = null;
    
    try { uri = event.dataTransfer.getData("text/uri-list"); }
    catch (ex) { alert(ex); }
    
    if (uri) {
        uri = uri.toString();
        uri = uri.replace(/[^a-z0-9.:\/\S!@#$%^&*()_-+=;\'\",?|]/gi,"\n");
        uri = uri.replace(/file:\/\/localhost/gi,'');
        uri = uri.replace(/%20/gi,' ');
        uri = uri.split("\n");
        
        
        for(i = 0; i < uri.length; i++) {
            newSet = uri[i];
            
            if(newSet.charAt(newSet.length-1) == '/')
                newSet = newSet.substring(0, newSet.length-1);
            else {
                alert("Invalid light set \"" + fileFromPath(newSet) + "\".");
                document.getElementById('controls').style.background = "url('controls.png')";
                return;
            }
            widget.system("cp -R \"" + newSet + "\" ~/Library/Application\\ Support/Festive\\ Lights/Lightsets/", null);
            newSet = fileFromPath(newSet);
        }
    }
   
    if(!switchLightSet(newSet))
    {
        newSetet = newSet.replace(' ', '\\ ');
        widget.system("rm -r ~/Library/Application\\ Support/Festive\\ Lights/Lightsets/" + newSet, null);
    } else
        loadLightSets();
    
    
    setPopupSelection(document.getElementById('set'), set);

    document.getElementById('controls').style.background = "url('controls.png')";

    event.stopPropagation();
    event.preventDefault();
}

function dragenter (event)
{
    document.getElementById('controls').style.background = "url('controls_drop.png')";
    event.stopPropagation();
    event.preventDefault();
}

function dragleave (event)
{
    document.getElementById('controls').style.background = "url('controls.png')";
    event.stopPropagation();
    event.preventDefault();
}

function ensureDirectoryExists(path)
{
	exists = (widget.system("ls " + path, null).errorString == "");

	if(!exists)
	{
		widget.system("mkdir -p " + path, null);
		return false;
	}	
	return true;
}

function pathExists(path)
{
	error = widget.system("ls " + path, null).errorString;
	return (error == null);
}

function fileFromPath(path)
{
    file = path.split("/");
    
    if(file[file.length-1] != "")
        return file[file.length-1];
    else
        return file[file.length-2];
}

if (window.widget)
{
   widget.onshow = onshow;
   widget.onhide = onhide;
}

function onshow () {
    visible = true;
    checkForUpdate();
}

function onhide () {
    visible = false;
}

function handleUpdateResponse()
{
	if(updateReq.readyState == 4 && updateReq.status == 200)
    {
    	latestVersion = updateReq.responseText;
		if(latestVersion)
		{
			if (version < latestVersion) {
				document.getElementById('im_tag').src = 'update_tag.png';
			}
		}
	}
}

function checkForUpdate()
{
    updateReq = new XMLHttpRequest();
    url = "http://www.interdimensionmedia.com/widgets/widgetversion.php?widget=" + name;
	updateReq.open("GET", url, true);
	updateReq.onreadystatechange = handleUpdateResponse;
	updateReq.send(null);
}